/**
 * @file   utils.h
 * @brief  Misc. functions to format strings, extract or push data, etc.
 * @author Alejandro Álvarez Ayllón <aalvarez@cern.ch>
 */
#ifndef UTILS_H_
#define	UTILS_H_

#include <dmlite/c/dmlite.h>
#include <dmlite/c/utils.h>
#include <mod_dav.h>

#define DAV_DPM_RFC2068 0
#define DAV_DPM_RFC3339 1

/**
 * Create a dav error with proper error message and HTTP status.
 * @param r         The request thar originated the error.
 * @param ctx       If not NULL, the error string/code will be retrieved from it.
 * @param http_code If specified, this will be used instead of mapping err_no.
 * @param format    Additional string to print (format).
 * @return          A initialized dav_error instance. Must not be freed.
 */
dav_error* dav_shared_new_error(request_rec *r, dmlite_context *ctx, int http_code,
                                const char *format, ...);

/**
 * Return a string with the date-time formated.
 * @param buffer Where to put the formated date.
 * @param maxlen The size of the buffer.
 * @param tstamp The timestamp.
 * @param format Format flag (DAV_DPM_RFC2068 or DAV_DPM_RFC3339).
 * @return       The length of the printed string.
 */
int dav_shared_format_datetime(char *buffer, size_t maxlen, time_t tstamp,
                               unsigned format);

/**
 * Add a response to the walk resource.
 * @param wres  The walk resource.
 * @param error The error to add.
 */
void dav_shared_add_response(dav_walk_resource *wres, dav_error *error);

/**
 * Parses a query (everything after a ?) and initializes a table with it.
 * @param pool  The pool to use.
 * @param query The query to parse.
 * @param size  The number of elements found.
 * @return      A hash table with key=value.
 */
apr_table_t *dav_shared_parse_query(apr_pool_t *pool, const char *query, unsigned *size);

/**
 * Generates a printable version of the filemode.
 * @param str  Where to put the string. Must have 9+1 bytes.
 * @param mode The mode.
 * @return     str.
 */
char *dav_shared_mode_str(char *str, mode_t mode);

/**
 * Generates a printable version of the size.
 * @param str  Where to put the string.
 * @param max  Max length.
 * @param size The size in bytes.
 * @return     str.
 */
char *dav_shared_size_str(char *str, size_t max, size_t size);

/**
 * Builds an URL from a url struct
 * @param pool           The pool for the memory
 * @param uri            The struct
 * @param default_scheme The default scheme to use (http or https)
 * @param default_port   Use this port if the scheme is not http or https
 * @return               A http URL.
 */
char *dav_shared_build_url(apr_pool_t *pool, dmlite_url *url,
        const char* default_scheme, unsigned default_port);

/**
 * Builds an URL using the aggregation data (rejected, and so forth)
 */
char *dav_shared_build_aggregation_url(apr_pool_t* pool,
                                       int n_replicas,
                                       const char** replicas,
                                       int64_t* replicas_ids,
                                       const char* forbidden_str,
                                       const char* notfound_str,
                                       int64_t forbidden_id,
                                       int64_t notfound_id);

/*
 * Return 1 if the requests specify that it accepts "content" mimetypes,
 * 0 otherwise.
 * (i.e. dav_shared_request_acceps(r, "application/metalink+xml")
 */
int dav_shared_request_accepts(request_rec* r, const char* content);

/* Utility functions to wrap dmlite free functions */
apr_status_t dav_shared_context_free(void*);
apr_status_t dav_shared_dict_free(void*);
apr_status_t dav_shared_fclose(void*);

#endif	/* UTILS_H_ */
